package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.base.StateConstant;
import com.example.afrishop_v3.enums.OrderStatusEnum;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.example.afrishop_v3.models.OrderRequest;
import com.example.afrishop_v3.models.TbCfCartRecordR;
import com.example.afrishop_v3.models.TbCfCoupon;
import com.example.afrishop_v3.models.TbCfOrder;
import com.example.afrishop_v3.repository.TbCfCartRecordRRepository;
import com.example.afrishop_v3.repository.TbCfCouponRepository;
import com.example.afrishop_v3.repository.TbCfItemOrderRepository;
import com.example.afrishop_v3.repository.TbCfOrderRepository;
import com.example.afrishop_v3.security.services.AuthenticationUser;
import com.example.afrishop_v3.util.IdUtil;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;

import static org.springframework.data.domain.Sort.Order.asc;
import static org.springframework.data.domain.Sort.Order.desc;

@RestController
@RequestMapping("/order")
public class OrderController {
    private final TbCfOrderRepository repository;
    private final TbCfCartRecordRRepository cartRepository;
    private final TbCfCouponRepository couponRepository;
    private final TbCfItemOrderRepository itemOrderRepository;
    private final AuthenticationUser user;

    public OrderController(TbCfOrderRepository repository, TbCfCartRecordRRepository cartRepository, TbCfCouponRepository couponRepository, TbCfItemOrderRepository itemOrderRepository, AuthenticationUser user) {
        this.repository = repository;
        this.cartRepository = cartRepository;
        this.couponRepository = couponRepository;
        this.itemOrderRepository = itemOrderRepository;
        this.user = user;
    }


    @PostMapping("/settle")
    public Result<TbCfOrder> settleAccount(@RequestBody String[] ids,@RequestParam(value = "toitableId", required = false) String toitableId) {
        //String userId = user.userId();
        List<TbCfCartRecordR> allByUserId = cartRepository.findAllByCartRecordIdIn(ids);

        if( allByUserId.isEmpty() ){
            return new Result<>(ResultCodeEnum.VALIDATE_ERROR.getCode(),"There are no items in the shopping cart");
        }

        TbCfOrder order = new TbCfOrder();


        order.setCouponId(toitableId);

        if( toitableId != null && !toitableId.isEmpty()){
            Optional<TbCfCoupon> byId = couponRepository.findById(toitableId);
            byId.ifPresent(order::setCoupon);
        }


        order.getItemOrderListFromCartList(allByUserId);



        return new Result<>(order);
    }



    @PostMapping("/place")
    public Result<TbCfOrder> placeOrder(@RequestBody OrderRequest tbCfOrder,
                             @RequestParam(value = "toitableId", required = false) String toitableId,
                             @RequestParam(value = "itemId", required = false) String itemId,
                             @RequestParam(value = "web", required = false) String web) throws IOException, URISyntaxException, ExecutionException, InterruptedException, TimeoutException {
        String userId = user.userId();

        int v_code = ResultCodeEnum.VALIDATE_ERROR.getCode();

        if( tbCfOrder == null)
            return new Result<>(v_code,"Empty body");

        if( tbCfOrder.getIds() == null || tbCfOrder.getIds().length <= 0)
            return new Result<>(v_code,"Empty body");

        String addressId = tbCfOrder.getDeliveryAddressId();

        if( addressId == null || addressId.isEmpty() ) return new Result<>(v_code,"Address id is required");


        List<TbCfCartRecordR> allByUserId = cartRepository.findAllByCartRecordIdIn(tbCfOrder.getIds());

        if( allByUserId.isEmpty() ){
            return new Result<>(v_code,"There are no items in the shopping cart");
        }


        TbCfOrder order = new TbCfOrder();
        order.setOrderId(IdUtil.createIdbyUUID());



        order.setCouponId(toitableId);

        if( toitableId != null && !toitableId.isEmpty()){
            Optional<TbCfCoupon> byId = couponRepository.findById(toitableId);
            byId.ifPresent(order::setCoupon);
        }


        order.setUserId(userId);

        if ("web".equals(web)) {
            order.setOrderSource(3);
        } else {
            order.setOrderSource(1);
        }


        order.getItemOrderListFromCartList(allByUserId);

        TbCfOrder save = repository.save(order);

        //order.getItemOrderList().forEach(itemOrderRepository::save);

        List<String> collect = allByUserId.stream().map(TbCfCartRecordR::getCartRecordId).collect(Collectors.toList());

        String[] strings = collect.toArray(new String[]{});
        cartRepository.deleteAllByCartRecordIdIn(strings);


        // implementation of coupon use

        return new Result<>(save);
    }
    @GetMapping
    public Result getUserOrderList(@RequestParam(value = "num", defaultValue = "0") Integer pageNum,
                                             @RequestParam(value = "size", defaultValue = "20") Integer pageSize,
                                             @RequestParam(value = "orderStatus", required = false) Integer orderStatus,
                                             @RequestParam(value = "sort", defaultValue = "desc") String sort,
                                             @RequestParam(value = "name", required = false) String name
    ) {
        Page<TbCfOrder> list = repository.findAllByUserId(user.userId(), PageRequest.of(pageNum, pageSize, sort(sort)));

        return new Result<>(list);
    }



    @GetMapping("/cancelOrder")
    public Result cancelOrder(@RequestParam("orderId") String orderId,
                              @RequestParam("reason") String reason) {
        Optional<TbCfOrder> byId = repository.findById(orderId);

        if( byId.isPresent() ){
            TbCfOrder order = byId.get();
            order.setOrderStatus(OrderStatusEnum.CLOSE.getValue());
            order.setRemarkInfo(reason);

            repository.save(order);

            return new Result();
        }


        return new Result<>(ResultCodeEnum.SERVICE_ERROR.getCode(),"Order not found!");
    }

    @DeleteMapping("/{orderId}")
    public Result deleteOrder(@PathVariable("orderId") String orderId){

        Optional<TbCfOrder> byId = repository.findById(orderId);

        if( byId.isPresent() ){
            TbCfOrder order = byId.get();

            order.setEnableFlag(StateConstant.INVALID);

            System.out.println(orderId);

            repository.save(order);

        }

        return new Result(ResultCodeEnum.ILLEGAL_ARGUMENT.getCode(),"");
    }

    private Sort sort(String order){
        String col = "orderTime";
        return Sort.by( "desc".equals(order) ? desc(col) : asc(col));
    }

}
